using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using System.Reflection;
using System;
using System.Text;
using UnityEngine.Networking;
using System.Security.Cryptography;
using TTSDK;

namespace Nebula
{
    public class FYHelper
    {
        /// <summary>
        /// model转Dictionary
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="obj"></param>
        /// <returns></returns>
        public static Dictionary<string, string> ModelToDic<T>(T obj)
        {
            Dictionary<string, string> map = new Dictionary<string, string>();
            if (obj == null)
            {
                return map;
            }
            Type t = obj.GetType();
            FieldInfo[] list = t.GetFields();
            foreach (var item in list)
            {
                string name = item.Name;
                object value = item.GetValue(obj);
                if (value != null)
                {
                    map.Add(name, value.ToString());
                }
                else
                {
                    map.Add(name, "");
                }
            }
            return map;
        }


        /// <summary>
        /// 返回大写的URL encode
        /// 特殊字符：+-*/%=<>()[]{}#@$&|\~^_:;!?,."`§±~ %·【】”；：，《》、
        /// URLEncode结果：%2B-%2A%2F%25%3D%3C%3E%28%29%5B%5D%7B%7D%23%40%24%26%7C%5C~%5E_%3A%3B%21%3F%2C.%22%60%C2%A7%C2%B1~%20%25%C2%B7%E3%80%90%E3%80%91%E2%80%9D%EF%BC%9B%EF%BC%9A%EF%BC%8C%E3%80%8A%E3%80%8B%E3%80%81
        /// </summary>
        /// <param name="str"></param>
        /// <returns></returns>
        public static string URLEncode(string str)
        {
            StringBuilder builder = new StringBuilder();
            foreach (char c in str)
            {
                if (UnityWebRequest.EscapeURL(c.ToString()).Length > 1)
                {
                    builder.Append(UnityWebRequest.EscapeURL(c.ToString()).ToUpper());
                }
                else if (c.ToString().Equals(" "))
                {
                    builder.Append("%20");
                }
                else if (c.ToString().Equals("*"))
                {
                    builder.Append("%2A");
                }
                else if (c.ToString().Equals("("))
                {
                    builder.Append("%28");
                }
                else if (c.ToString().Equals(")"))
                {
                    builder.Append("%29");
                }
                else
                {
                    builder.Append(c);
                }
            }
            return builder.ToString();
        }

        /// <summary>
        /// 获取Md5加密串
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        public static string GetMD5(String input)
        {
            if (input == null)
            {
                return null;
            }

            MD5 md5Hash = MD5.Create();

            // 将输入字符串转换为字节数组并计算哈希数据
            byte[] data = md5Hash.ComputeHash(Encoding.UTF8.GetBytes(input));

            // 创建一个 Stringbuilder 来收集字节并创建字符串
            StringBuilder sBuilder = new StringBuilder();

            // 循环遍历哈希数据的每一个字节并格式化为十六进制字符串
            for (int i = 0; i < data.Length; i++)
            {
                sBuilder.Append(data[i].ToString("x2"));
            }

            // 返回十六进制字符串
            return sBuilder.ToString();
        }

        /// <summary>
        /// 获取签名串
        /// </summary>
        /// <param name="param"></param>
        /// <returns></returns>
        public static string GetSign<T>(T model, string secret)
        {
            Dictionary<string, string> param = FYHelper.ModelToDic<T>(model);

            List<string> keys = new List<string>(param.Keys);
            keys.Sort();

            // 排除sign
            if (keys.Contains("sign"))
            {
                keys.Remove("sign");
            }

            StringBuilder sb = new StringBuilder();
            int index = 0;
            foreach (var key in keys)
            {
                sb.Append(key);
                sb.Append("=");
                sb.Append(param[key]);
                if (index < keys.Count - 1)
                {
                    sb.Append("&");
                }

                index++;
            }
            string str = FYHelper.URLEncode(sb.ToString()) + $"&{secret}";
            string sign = GetMD5(str);
            return sign;
        }

        /// <summary>
        /// 生成随机字符串
        /// </summary>
        /// <returns></returns>
        public static string RandomString(int length)
        {
            const string chars = "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
            var random = new System.Random();
            var result = new char[length];
            for (int i = 0; i < result.Length; i++)
            {
                result[i] = chars[random.Next(chars.Length)];
            }
            return new string(result);
        }

        public static string DeviceId()
        {
            string key = "NEBULA_SDK_CORE_DEVICE_ID";

            string deviceId = TT.PlayerPrefs​.GetString(key);

            if (String.IsNullOrEmpty(deviceId))
            {
                string temp = $"{RandomString(8)}{System.DateTimeOffset.UtcNow.ToUnixTimeSeconds()}";
                deviceId = GetMD5(temp);
                TT.PlayerPrefs​.SetString(key, deviceId);
            }
            return deviceId;
        }

        public static string StartupId()
        {
            string temp = $"{RandomString(8)}{System.DateTimeOffset.UtcNow.ToUnixTimeSeconds()}";
            string startupId = GetMD5(temp);
            return startupId;
        }

        /// <summary>
        /// 格式化时间字符串，精确到毫秒
        /// </summary>
        /// <returns></returns>
        public static string GetCurrentTimeMilliseconds()
        {
            // 获取当前时间
            DateTime now = DateTime.Now;

            // 格式化时间字符串，精确到毫秒
            string formattedTime = now.ToString("yyyy-MM-dd HH:mm:ss:fff");

            return formattedTime;
        }

        /// <summary>
        /// 格式化时间字符串，精确到秒
        /// </summary>
        /// <returns></returns>
        public static string GetCurrentTime()
        {
            // 获取当前时间
            DateTime now = DateTime.Now;

            // 格式化时间字符串，精确到秒
            string formattedTime = now.ToString("yyyy-MM-dd HH:mm:ss");

            return formattedTime;
        }

        /// <summary>
        /// 获取当前UTC时间的时间戳，精确到毫秒
        /// </summary>
        /// <returns></returns>
        public static long GetTimestampMilliseconds()
        {
            // 获取当前UTC时间的时间戳，精确到毫秒
            return DateTimeOffset.UtcNow.ToUnixTimeMilliseconds();
        }
    }
}