using System;
using System.Text;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.Networking;

namespace Nebula
{
    public class FYHttpHelper : MonoBehaviour
    {
        private static FYHttpHelper instance;

        public static FYHttpHelper Instance
        {
            get
            {
                if (instance == null)
                {
                    GameObject go = new GameObject("FYHttpHelper");
                    instance = go.AddComponent<FYHttpHelper>();
                }
                return instance;
            }
        }

        #region Get请求
        /// <summary>
        /// Get请求
        /// </summary>
        /// <param name="url"></param>
        /// <param name="actionResult"></param>
        public void Get(string url, Dictionary<string, string> formFields, Action<bool, string> actionResult = null)
        {

            if (formFields != null)
            {
                url = $"{url}?{GetParams(formFields)}";
            }

            StartCoroutine(_Get(url, actionResult));
        }

        private IEnumerator _Get(string url, Action<bool, string> action)
        {
            using (UnityWebRequest request = UnityWebRequest.Get(url))
            {
                request.timeout = 10;
                yield return request.SendWebRequest();

                string resstr = "";
                bool isSuccess = IsRequestSuccess(request);
                if (isSuccess)
                {
                    while (!request.downloadHandler.isDone)
                        yield return null;
                    resstr = request.downloadHandler.text;
                }
                else
                {
                    if (request.result == UnityWebRequest.Result.ConnectionError)
                    {
                        resstr = "网络异常";
                    }
                    else if (request.result == UnityWebRequest.Result.ProtocolError)
                    {
                        resstr = $"服务器异常({request.responseCode})";
                    }
                    else
                    {
                        resstr = request.error;
                    }
                }

                Log(url, "", "GET", isSuccess, resstr);

                if (action != null)
                {
                    action(isSuccess, resstr);
                }
            }
        }
        #endregion


        #region POST请求
        /// <summary>
        /// POST请求
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="url"></param>
        /// <param name="model"></param>
        /// <param name="actionResult"></param>
        /// <param name="headers"></param>
        public void Post<T>(string url, T model, Action<bool, string> actionResult = null, Dictionary<string, string> headers = null)
        {
            StartCoroutine(_Post(url, PostParams(model), actionResult, headers));
        }

        private IEnumerator _Post(string url, string data, Action<bool, string> action, Dictionary<string, string> headers)
        {
            using (UnityWebRequest request = new UnityWebRequest(url, "POST"))
            {
                request.uploadHandler = new UploadHandlerRaw(Encoding.UTF8.GetBytes(data));
                request.SetRequestHeader("content-type", "application/json;charset=utf-8");
                if (headers != null)
                {
                    foreach (var key in headers.Keys)
                    {
                        if (headers[key] != null)
                        {
                            request.SetRequestHeader(key, headers[key]);
                        }

                    }
                }
                request.downloadHandler = new DownloadHandlerBuffer();
                request.timeout = 10;
                yield return request.SendWebRequest();

                string resstr = "";
                bool isSuccess = IsRequestSuccess(request);
                if (isSuccess)
                {
                    while (!request.downloadHandler.isDone)
                        yield return null;

                    resstr = request.downloadHandler.text;
                }
                else
                {
                    if (request.result == UnityWebRequest.Result.ConnectionError)
                    {
                        resstr = "网络异常";
                    }
                    else if (request.result == UnityWebRequest.Result.ProtocolError)
                    {
                        resstr = $"服务器异常({request.responseCode})";
                    }
                    else
                    {
                        resstr = request.error;
                    }
                }

                Log(url, data, "POST", isSuccess, resstr);

                if (action != null)
                {
                    action(isSuccess, resstr);
                }
            }
        }
        #endregion





        /// <summary>
        /// 是否请求成功
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        private bool IsRequestSuccess(UnityWebRequest request)
        {
            return request.result == UnityWebRequest.Result.Success;
        }

        /// <summary>
        /// 打印日志
        /// </summary>
        /// <param name="url"></param>
        /// <param name="param"></param>
        /// <param name="method"></param>
        /// <param name="isSuccess"></param>
        /// <param name="result"></param>
        private void Log(string url, string param, string method, bool isSuccess, string result)
        {
            // 日志上报的请求不打印
            if (url.Contains("feishu-develop-tracker"))
            {
                return;
            }
            Debug.Log($"FYHttpHelper>>url:{url}>>param:{param}>>method:{method}>>isSuccess:{isSuccess}>>result:{result}");
        }

        /// <summary>
        /// 获取Get参数串
        /// </summary>
        /// <param name="formFields"></param>
        /// <returns></returns>
        private string GetParams(Dictionary<string, string> formFields)
        {
            StringBuilder sb = new StringBuilder();
            int index = 0;
            foreach (var key in formFields.Keys)
            {
                sb.Append(key);
                sb.Append("=");
                sb.Append(formFields[key]);
                if (index != formFields.Keys.Count - 1)
                {
                    sb.Append("&");
                }
                index++;
            }
            return sb.ToString();
        }

        /// <summary>
        /// 获取POST参数串
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="model"></param>
        /// <returns></returns>
        private string PostParams<T>(T model)
        {
            string json = JsonUtility.ToJson(model);
            return json;
        }
    }
}